@@ -1,23 +1,22 @@
 ## Here the main tCorpus class is created
 ## Additional methods are specified in the matching r files (e.g. dtm method in document_term_matrix.r)
-## The additional methods should only use the public functions of the tCorpus class (so avoid private$)
-## This way, we can more easily change or add backends in the future.
 
-## note that this file has to be named 1_*, because r files are processed in order by name,
+## The additional methods should always use only the accessor methods (so never use self$tokens and self$meta directly).
+## This way, all validations, assumptions and optimalizations stay intact
+
+## this file has to be named 1_*, because in building the package the r files are processed in order by name,
 ## and the tCorpus class has to be created before the additional methods
 
 tCorpus <- R6::R6Class("tCorpus",
    cloneable=FALSE,
 
    private = list(
-     .data = NULL,
-     .meta = NULL,
      .p = list(),
 
      sync = function(){
-       doc_ids = intersect(unique(private$.data$doc_id), unique(private$.meta$doc_id))
-       private$.data = private$.data[list(doc_ids),]
-       private$.meta = private$.meta[list(doc_ids),]
+       doc_ids = intersect(unique(self$tokens$doc_id), unique(self$meta$doc_id))
+       self$tokens = self$tokens[list(doc_ids),]
+       self$meta = self$meta[list(doc_ids),]
        private$set_keys()
      },
 
@@ -26,14 +25,32 @@ tCorpus <- R6::R6Class("tCorpus",
      },
 
      set_keys = function(){
-       if (!identical(key(private$.data), c('doc_id', 'token_id'))) setkey(private$.data, 'doc_id', 'token_id')
-       if (!identical(key(private$.meta), c('doc_id'))) setkey(private$.meta, 'doc_id')
+       if (!identical(key(self$tokens), c('doc_id', 'token_id'))) setkey(self$tokens, 'doc_id', 'token_id')
+       if (!identical(key(self$meta), c('doc_id'))) setkey(self$meta, 'doc_id')
      },
 
      droplevels = function(){
-       private$.data = base::droplevels(private$.data)
-       private$.meta = base::droplevels(private$.meta)
+       self$tokens = base::droplevels(self$tokens)
+       self$meta = base::droplevels(self$meta)
        invisible(self)
+     },
+
+     select_rows = function(selection, keep_meta=F) {
+       selection = safe_selection(self$tokens, selection)
+       self$tokens = subset(self$tokens, selection)
+       if (!keep_meta) {
+         self$meta = self$meta[as.character(unique(self$tokens$doc_id)),,nomatch=0]
+         self$meta$doc_id = as.character(self$meta$doc_id)
+       }
+       private$set_keys()
+     },
+
+     select_meta_rows = function(selection, keep_data=F) {
+       selection = safe_selection(self$meta, selection)
+       self$meta = subset(self$meta, selection)
+       self$meta$doc_id = as.character(self$meta$doc_id)
+       if (!keep_data) self$tokens = self$tokens[as.character(unique(self$meta$doc_id)),,nomatch=0]
+       private$set_keys()
      }
 
    ),
@@ -41,9 +58,43 @@ tCorpus <- R6::R6Class("tCorpus",
    public = list(
      help = function() ?tCorpus,
 
-     initialize = function(data, meta) {
-       private$.data = data.table(data)
-       private$.meta = data.table(meta)
+     tokens = NULL,
+     meta = NULL,
+
+     validate = function() {
+        self$validate_tokens()
+        self$validate_meta()
+     },
+     validate_tokens = function() {
+        cnames = self$names
+        e = c()
+        if (!'doc_id' %in% cnames) e = c(e, '\t- tokens data does not have a "doc_id" column. You can try to fix the data in $tokens')
+        if (!'token_id' %in% cnames) e = c(e, '\t- tokens data does not have "token_id" column. You can try to fi the data in $tokens')
+        if (any(grepl('^\\.', cnames))) e = c(e, '\t- column names in a tCorpus cannot start with a dot. You can change the column names in $tokens or with the $set_name method')
+        if (length(e) > 0) {
+          e_message = paste0('TOKEN DATA BROKEN:\n', paste(e, collapse='\n'))
+          stop(e_message)
+        }
+
+        if (!identical(data.table::key(self$tokens), c('doc_id','token_id'))) data.table::setkeyv(self$tokens, c('doc_id','token_id'))
+     },
+
+     validate_meta = function() {
+       cnames = self$meta_names
+       e = c()
+       if (!'doc_id' %in% cnames) e = c(e, '\t- meta data does not have a "doc_id" column. You can try to fix the data in $meta')
+       if (any(grepl('^\\.', cnames)))  e = c(e, '\t- column names in a tCorpus cannot start with a dot. You can change the column names in $meta or with the $set_meta_name method')
+       if (length(e) > 0) {
+         e_message = paste0('META DATA BROKEN:\n', paste(e, collapse='\n'))
+         stop(e_message)
+       }
+
+       if (!identical(data.table::key(self$meta), 'doc_id')) data.table::setkeyv(self$meta, 'doc_id')
+     },
+
+     initialize = function(tokens, meta) {
+       self$tokens = data.table(tokens)
+       self$meta = data.table(meta)
        private$set_keys()
      },
 
@@ -52,99 +103,109 @@ tCorpus <- R6::R6Class("tCorpus",
      },
 
      copy = function(){
-       tCorpus$new(data = data.table::copy(private$.data),
-                   meta = data.table::copy(private$.meta))
+       tCorpus$new(tokens = data.table::copy(self$tokens),
+                   meta = data.table::copy(self$meta))
      },
 
 ## SHOW/GET DATA METHODS ##
-    get = function(columns=NULL, keep_df=F, as.df=F, subset=NULL, doc_id=NULL, token_id=NULL, safe_copy=T) {
-      if (class(substitute(subset)) %in% c('call', 'name')) subset = self$eval(substitute(subset), parent.frame())
-      if (!is.null(doc_id) & !is.null(subset)) stop('Cannot filter by subset and doc_ids at the same time')
-      if (is.null(doc_id) & !is.null(token_id)) stop('token_id can only be given in pairs with doc_id')
+    get = function(columns=NULL, keep_df=F, as.df=F, subset=NULL, doc_id=NULL, token_id=NULL, copy=F) {
+      self$validate_tokens()
 
-      if (is.null(doc_id) & is.null(subset)) {
+      if (class(substitute(subset)) %in% c('call', 'name')) subset = self$eval(substitute(subset), parent.frame())
+      if (!is.null(doc_id) && !is.null(subset)) stop('Cannot filter by subset and doc_ids at the same time')
+      if (is.null(doc_id) && !is.null(token_id)) stop('token_id can only be given in pairs with doc_id')
+
+      if (is.null(doc_id) && is.null(subset)) {
         if (is.null(columns)) {
-          d = private$.data
+          d = self$tokens
         } else {
-          d = if (length(columns) > 1 | keep_df) private$.data[,columns,with=F] else private$.data[[columns]]
+          d = self$tokens[,columns,with=F]
         }
       } else {
-        if (is.null(columns)) columns = colnames(private$.data)
-        if (!is.null(subset)) d = if (length(columns) > 1 | keep_df) private$.data[subset,columns,with=F] else private$.data[[columns]][subset]
+        if (is.null(columns)) columns = colnames(self$tokens)
+        if (!is.null(subset)) self$tokens[subset,columns,with=F]
         if (!is.null(doc_id)) {
           if (is.null(token_id)) {
             positions = list(doc_ids = as.character(doc_id))
-            d = if (length(columns) > 1 | keep_df) private$.data[positions,columns,with=F] else private$.data[positions, columns, with=F][[columns]]
+            d = self$tokens[positions,columns,with=F]
           } else {
             if (!length(doc_id) == length(token_id)) stop('token_id can only be given in pairs with doc_id')
             positions = list(doc_ids = as.character(doc_id), token_is = as.numeric(token_id))
-            d = if (length(columns) > 1 | keep_df) private$.data[positions,columns,with=F] else private$.data[positions, columns, with=F][[columns]]
+            d = self$tokens[positions,columns,with=F]
           }
         }
       }
 
+      if (ncol(d) == 1 && !keep_df) d = d[,1][[1]]
       if (as.df) d = as.data.frame(d)
-      if(safe_copy) data.table::copy(d) else d
+      if(copy) data.table::copy(d) else d
     },
 
     get_levels = function(column){
-      if (!methods::is(private$.data[[column]], 'factor')) stop(sprintf('"%s" is not a factor', column))
-      data.table::copy(levels(private$.data[[column]]))
+      if (!column %in% self$names) stop(sprintf('"%s" column does not exists in tokens', column))
+      if (!methods::is(self$tokens[[column]], 'factor')) stop(sprintf('"%s" is not a factor', column))
+      data.table::copy(levels(self$tokens[[column]]))
     },
 
-    get_meta = function(columns=NULL, keep_df=F, as.df=F, per_token=F, subset=NULL, doc_id=NULL, safe_copy=T) {
+    get_meta = function(columns=NULL, keep_df=F, as.df=F, per_token=F, subset=NULL, doc_id=NULL, copy=T) {
+      self$validate()
+
       if (class(substitute(subset)) %in% c('call', 'name')) subset = self$eval_meta(substitute(subset), parent.frame())
-      if (!is.null(doc_id) & !is.null(subset)) stop('Cannot filter by subset and doc_ids at the same time')
+      if (!is.null(doc_id) && !is.null(subset)) stop('Cannot filter by subset and doc_ids at the same time')
 
 
-      if (is.null(doc_id) & is.null(subset)) {
+      if (is.null(doc_id) && is.null(subset)) {
         if (is.null(columns)) {
-          d = private$.meta
+          d = self$meta
         } else {
-          d = if (length(columns) > 1 | keep_df) private$.meta[,columns,with=F] else private$.meta[[columns]]
+          d = self$meta[,columns,with=F]
         }
       } else {
-        if (is.null(columns)) columns = colnames(private$.meta)
+        if (is.null(columns)) columns = colnames(self$meta)
         if (!is.null(doc_id)) {
           doc_ids = as.character(doc_id)
-          d = if (length(columns) > 1 | keep_df) private$.meta[list(doc_ids),columns,with=F] else private$.meta[list(doc_ids), columns, with=F][[columns]]
+          d = self$meta[list(doc_ids),columns,with=F]
         }
-        if (!is.null(subset)) d = if (length(columns) > 1 | keep_df) private$.meta[subset,columns,with=F] else private$.meta[[columns]][subset]
+        if (!is.null(subset)) self$meta[subset,columns,with=F]
       }
 
+      if (ncol(d) == 1 && !keep_df) d = d[,1][[1]]
       if (as.df) d = as.data.frame(d)
       if (per_token) {
-        exp_i = match(private$.data$doc_id, private$.meta$doc_id)
+        exp_i = match(self$tokens$doc_id, self$meta$doc_id)
         d = if (methods::is(d, 'data.frame')) d[exp_i,,drop=!keep_df & !is.null(columns)] else d[exp_i]
       }
-      if(safe_copy) data.table::copy(d) else d
+      if(copy) data.table::copy(d) else d
     },
 
     get_meta_levels = function(column){
-      if (!methods::is(private$.meta[[column]], 'factor')) stop(sprintf('"%s" is not a factor', column))
-      data.table::copy(levels(private$.meta[[column]]))
+      if (!column %in% self$meta_names) stop(sprintf('"%s" column does not exists in meta', column))
+      if (!methods::is(self$meta[[column]], 'factor')) stop(sprintf('"%s" is not a factor', column))
+      data.table::copy(levels(self$meta[[column]]))
     },
 
      context = function(context_level = c('document','sentence'), with_labels=T){
        get_context(self, context_level = context_level, with_labels=with_labels)
      },
 
-     eval = function(x, enclos=parent.frame()) eval(x, private$.data, enclos),
-     eval_meta = function(x, enclos=parent.frame()) eval(x, private$.meta, enclos),
+     eval = function(x, enclos=parent.frame()) eval(x, self$tokens, enclos),
+     eval_meta = function(x, enclos=parent.frame()) eval(x, self$meta, enclos),
+
+    get_token_id = function(doc_id=NULL, token_id=NULL, subset=NULL, subset_meta=NULL, window=NULL, inverse=F){
+      self$validate()
 
-     token_id = function(doc_id=NULL, token_id=NULL, subset=NULL, subset_meta=NULL, window=NULL, inverse=F){
       if (class(substitute(subset)) %in% c('call', 'name')) subset = self$eval(substitute(subset), parent.frame())
       if (class(substitute(subset_meta)) %in% c('call', 'name')) subset_meta = self$eval_meta(substitute(subset_meta), parent.frame())
-      if (is.null(doc_id) & !is.null(token_id)) stop('token_id can only be given in pairs with doc_id')
+      if (is.null(doc_id) && !is.null(token_id)) stop('token_id can only be given in pairs with doc_id')
 
       ## enable subset to be called from a character string. (e.g. used in search_features)
-      if(methods::is(subset, 'character')) subset_meta = eval(parse(text=subset_meta), private$.meta, parent.frame())
-      if(methods::is(subset_meta, 'character')) subset_meta = eval(parse(text=subset_meta), private$.meta, parent.frame())
+      if(methods::is(subset, 'character')) subset_meta = eval(parse(text=subset_meta), self$meta, parent.frame())
+      if(methods::is(subset_meta, 'character')) subset_meta = eval(parse(text=subset_meta), self$meta, parent.frame())
 
       i = NULL
       if (!is.null(doc_id)) {
         pos = if (is.null(token_id)) list(doc_id) else list(doc_id, token_id)
-        i = private$.data[pos, which=T]
+        i = self$tokens[pos, which=T]
       }
       if (!is.null(subset)) {
         subset[is.na(subset)] = F
@@ -153,7 +214,7 @@ tCorpus <- R6::R6Class("tCorpus",
       if (!is.null(subset_meta)) {
         subset_meta[is.na(subset_meta)] = F
         doc_ids = self$get_meta('doc_id')[subset_meta]
-        meta_i = private$.data[list(doc_ids), which=T]
+        meta_i = self$tokens[list(doc_ids), which=T]
         i = if (is.null(i)) meta_i else intersect(i, meta_i)
       }
 
@@ -164,50 +225,61 @@ tCorpus <- R6::R6Class("tCorpus",
 
 
 ## DATA MODIFICATION METHODS ##
+
+     set_special = function(token=NULL, lemma=NULL, POS=NULL, relation=NULL, parent=NULL) {
+       if(!is.null(token)) self$set_name(token, 'token')
+       if(!is.null(lemma)) self$set_name(lemma, 'lemma')
+       if(!is.null(POS)) self$set_name(POS, 'POS')
+       if(!is.null(relation)) self$set_name(relation, 'relation')
+       if(!is.null(parent)) self$set_name(parent, 'parent')
+     },
+
      set = function(column, value, subset=NULL, subset_value=T){
+       self$validate_tokens()
+
        if (column == 'doc_id') stop('Cannot change doc_id. If you want to change doc_id labels, you can overwrite $doc_id_levels.')
        if (class(substitute(subset)) %in% c('call', 'name')) subset = self$eval(substitute(subset), parent.frame())
        if (class(substitute(value)) %in% c('call', 'name')) value = self$eval(substitute(value), parent.frame())
        if (grepl('^\\.', column)) stop('column names in a tCorpus cannot start with a dot')
 
        if (!is.null(subset)){
-         if (subset_value & length(value) > 1) value = value[subset]
-         if (!column %in% colnames(private$.data)) {
-           private$.data[,(column) := NA]
+         if (subset_value && length(value) > 1) value = value[subset]
+         if (!column %in% colnames(self$tokens)) {
+           self$tokens[,(column) := NA]
            if (methods::is(value, 'factor')) {
-             private$.data[,(column) := fast_factor(private$.data[[column]])]
+             self$tokens[,(column) := fast_factor(self$tokens[[column]])]
            } else {
-             private$.data[,(column) := methods::as(private$.data[[column]], class(value))]
+             self$tokens[,(column) := methods::as(self$tokens[[column]], class(value))]
            }
          }
 
          if (column %in% c('sentence','token_id')) {  ## for position columns, first perform checks (inefficient, but this should be a rare case anyway)
             if (!methods::is(value, 'numeric')) stop('position column has to be numeric/integer')
             value = as.numeric(value)
-            mod = if ('sentence' %in% self$names) private$.data[,c('doc_id','sentence','token_id')] else private$.data[,c('doc_id','token_id')]
+            mod = if ('sentence' %in% self$names) self$tokens[,c('doc_id','sentence','token_id')] else self$tokens[,c('doc_id','token_id')]
             mod[subset, (column) := value]
             check_unique_rows(mod)
          }
 
          .value = value
-         private$.data[subset, (column) := .value]
+         self$tokens[subset, (column) := .value]
 
        } else {
          if (column %in% c('sentence','token_id')) {
            if (!methods::is(value, 'numeric')) stop('position column has to be numeric/integer')
            value = as.numeric(value)
-           mod = if ('sentence' %in% self$names) private$.data[,c('doc_id','sentence','token_id')] else private$.data[,c('doc_id','token_id')]
+           mod = if ('sentence' %in% self$names) self$tokens[,c('doc_id','sentence','token_id')] else self$tokens[,c('doc_id','token_id')]
            suppressWarnings(mod[, (column) := value])
            check_unique_rows(mod)
          }
          if (is.character(value)) value = fast_factor(value)  ## force the use of factors for character vectors.
          ## ugly suppress. Should look into why data.table give the (seemingly harmless) internal.selfref warning
-         suppressWarnings(private$.data[,(column) := value])
+         suppressWarnings(self$tokens[,(column) := value])
        }
 
-       ## strangely, the assign by reference in data.table sometimes adds NA as a factor level...
-       if (anyNA(levels(private$.data[[column]]))) {
-         private$.data[,(column) := fast_factor(private$.data[[column]])]
+       ## the assign by reference in data.table sometimes adds NA as a factor level...
+       if (anyNA(levels(self$tokens[[column]]))) {
+         self$tokens[,(column) := fast_factor(self$tokens[[column]])]
        }
 
        private$set_keys()
@@ -215,30 +287,33 @@ tCorpus <- R6::R6Class("tCorpus",
      },
 
      set_levels = function(column, levels) {
-       if (!methods::is(private$.data[[column]], 'factor')) stop(sprintf('"%s" is not a factor', column))
-       if (!length(levels) == length(levels(private$.data[[column]]))) stop('new levels of different length than current levels')
+       if (!column %in% self$names) stop(sprintf('"%s" column does not exists in tokens', column))
+       if (!methods::is(self$tokens[[column]], 'factor')) stop(sprintf('"%s" is not a factor', column))
+       if (!length(levels) == length(levels(self$tokens[[column]]))) stop('new levels of different length than current levels')
        if (column == 'doc_id') {
-         private$.meta$doc_id = levels[match(levels(private$.data$doc_id), private$.meta$doc_id)]
+         self$meta$doc_id = levels[match(levels(self$tokens$doc_id), self$meta$doc_id)]
        }
-       data.table::setattr(private$.data[[column]], 'levels', levels)
+       data.table::setattr(self$tokens[[column]], 'levels', levels)
      },
 
      delete_columns = function(cnames){
        protected_cols = intersect(self$names, c('doc_id', 'token_id'))
        if (any(protected_cols %in% cnames)) stop("The position columns doc_id and token_id cannot be deleted")
-       for (col in cnames) private$.data[,(col) := NULL]
+       for (col in cnames) self$tokens[,(col) := NULL]
        invisible(self)
      },
 
      set_name = function(oldname, newname) {
-       if (oldname %in% c('doc_id','sentence','token_id')) stop('The position columns (doc_id, sentence, token_id) cannot be set or changed')
+       if (oldname %in% c('doc_id','token_id')) stop('The position columns (doc_id, token_id) cannot be set or changed')
        if (grepl('^\\.', newname)) stop('column names in a tCorpus cannot start with a dot')
 
-       data.table::setnames(private$.data, oldname, newname)
+       data.table::setnames(self$tokens, oldname, newname)
        invisible(self)
      },
 
      set_meta = function(column, value, subset=NULL, subset_value=T){
+       self$validate_meta()
+
        if (column == 'doc_id') stop('Cannot change doc_id. If you want to change doc_id labels, you can overwrite $doc_id_levels.')
        if (class(substitute(subset)) %in% c('call', 'name')) subset = self$eval_meta(substitute(subset), parent.frame())
        if (class(substitute(value)) %in% c('call', 'name')) value = self$eval_meta(substitute(value), parent.frame())
@@ -246,74 +321,59 @@ tCorpus <- R6::R6Class("tCorpus",
        if (grepl('^\\.', column)) stop('column names in a tCorpus cannot start with a dot')
 
        if (!is.null(subset)){
-         if (subset_value & length(value) > 1) value = value[subset]
+         if (subset_value && length(value) > 1) value = value[subset]
 
-         if (!column %in% colnames(private$.meta)) {
-           private$.meta[[column]] = NA
+         if (!column %in% colnames(self$meta)) {
+           self$meta[[column]] = NA
            if (methods::is(value, 'factor')) {
-             private$.meta[[column]] = fast_factor(private$.meta[[column]])
+             self$meta[[column]] = fast_factor(self$meta[[column]])
            } else {
-             private$.meta[,(column) := methods::as(private$.meta[[column]], class(value))]
+             self$meta[,(column) := methods::as(self$meta[[column]], class(value))]
            }
          }
 
          .value = value
-         private$.meta[subset, (column) := .value]
+         self$meta[subset, (column) := .value]
 
        } else {
          ## ugly suppress. Should look into why data.table give the (seemingly harmless) internal.selfref warning
-         suppressWarnings(private$.meta[,(column) := value])
+         suppressWarnings(self$meta[,(column) := value])
        }
 
        ## strangely, the assign by reference in data.table sometimes adds NA as a factor level...
-       if (anyNA(levels(private$.meta[[column]]))) {
-         private$.meta[,(column) := fast_factor(private$.meta[[column]])]
+       if (anyNA(levels(self$meta[[column]]))) {
+         self$meta[,(column) := fast_factor(self$meta[[column]])]
        }
        invisible(self)
      },
 
      set_meta_levels = function(column, levels) {
-        if (!methods::is(private$.meta[[column]], 'factor')) stop(sprintf('"%s" is not a factor', column))
-        data.table::setattr(private$.meta[[column]], 'levels', levels)
+        if (!column %in% self$meta_names) stop(sprintf('"%s" column does not exists in meta', column))
+        if (!methods::is(self$meta[[column]], 'factor')) stop(sprintf('"%s" is not a factor', column))
+        data.table::setattr(self$meta[[column]], 'levels', levels)
       },
 
      delete_meta_columns = function(cnames){
         protected_cols = intersect(self$names, c('doc_id'))
         if (any(protected_cols %in% cnames)) stop('doc_id cannot be deleted')
-        for (col in cnames) private$.meta[,(col) := NULL]
+        for (col in cnames) self$meta[,(col) := NULL]
         invisible(self)
       },
 
      set_meta_name = function(oldname, newname) {
        if (oldname %in% c('doc_id')) stop('The doc_id column cannot be set or changed')
        if (grepl('^\\.', newname)) stop('column names in a tCorpus cannot start with a dot')
-       setnames(private$.meta, oldname, newname)
+       setnames(self$meta, oldname, newname)
        invisible(self)
      },
 
-      select_rows = function(selection, keep_meta=F) {
-        selection = safe_selection(private$.data, selection)
-        private$.data = subset(private$.data, selection)
-        if (!keep_meta) {
-          private$.meta = private$.meta[as.character(unique(private$.data$doc_id)),,nomatch=0]
-          private$.meta$doc_id = as.character(private$.meta$doc_id)
-        }
-        private$set_keys()
-      },
-
-      select_meta_rows = function(selection, keep_data=F) {
-        selection = safe_selection(private$.meta, selection)
-        private$.meta = subset(private$.meta, selection)
-        private$.meta$doc_id = as.character(private$.meta$doc_id)
-        if (!keep_data) private$.data = private$.data[as.character(unique(private$.meta$doc_id)),,nomatch=0]
-        private$set_keys()
-      },
-
      subset = function(subset=NULL, subset_meta=NULL, hits=NULL, window=NULL, copy=F, ...){
+       self$validate()
+
        if (class(substitute(subset)) %in% c('call', 'name')) subset = self$eval(substitute(subset), parent.frame())
        if (class(substitute(subset_meta)) %in% c('call', 'name')) subset_meta = self$eval_meta(substitute(subset_meta), parent.frame())
        if (copy) {
-         selfcopy = self$copy()$subset(subset=subset, subset_meta=subset_meta, window=window, copy=F)
+         selfcopy = self$copy()$subset(subset=subset, subset_meta=subset_meta, hits=hits, window=window, copy=F)
          return(selfcopy)
        }
 
@@ -330,23 +390,23 @@ tCorpus <- R6::R6Class("tCorpus",
        }
 
        ## subset the data, using different solutions if one or both subsets (data and meta) are used for optimalisation
-       if (!is.null(subset_meta) & !is.null(subset)) {
-         self$select_meta_rows(subset_meta, keep_data = T) ## if both subsets are used, first perform both without subseting the other, then match on doc_ids.
-         self$select_rows(subset, keep_meta = T)           ## (we cannot subset one before the other, because subset call's can contains vectors for which the rows should match)
+       if (!is.null(subset_meta) && !is.null(subset)) {
+         private$select_meta_rows(subset_meta, keep_data = T) ## if both subsets are used, first perform both without subseting the other, then match on doc_ids.
+         private$select_rows(subset, keep_meta = T)           ## (we cannot subset one before the other, because subset call's can contains vectors for which the rows should match)
          private$sync()
        }
-       if (!is.null(subset_meta) & is.null(subset)) {
-         self$select_meta_rows(subset_meta, keep_data = F)
+       if (!is.null(subset_meta) && is.null(subset)) {
+         private$select_meta_rows(subset_meta, keep_data = F)
        }
-       if (is.null(subset_meta) & !is.null(subset)) {
-         self$select_rows(subset, keep_meta = F)
+       if (is.null(subset_meta) && !is.null(subset)) {
+         private$select_rows(subset, keep_meta = F)
        }
 
        private$droplevels()
        invisible(self)
      },
 
-      subset_meta = function(subset=NULL, copy=F){
+     subset_meta = function(subset=NULL, copy=F){
         ## subset also has a subset_meta argument, but we add this for consistency with other _meta methods
         if (class(substitute(subset)) %in% c('call', 'name')) subset = self$eval_meta(substitute(subset), parent.frame())
         .subset = subset
@@ -354,20 +414,22 @@ tCorpus <- R6::R6Class("tCorpus",
       },
 
      aggregate = function(meta_cols=NULL, hits=NULL, feature=NULL, count=c('documents','tokens', 'hits'), wide=T){
+        self$validate()
+
         count = match.arg(count)
 
-        meta = data.table::copy(private$.meta)
+        meta = data.table::copy(self$meta)
         if (is.null(meta_cols)) {
           meta[,group := 'total']
           meta_cols = 'group'
         }
 
-        len = private$.data[, list(len=.N), by='doc_id']
+        len = self$tokens[, list(len=.N), by='doc_id']
         meta = merge(meta, len, by='doc_id', all.x=T)
         d = meta[, list(N=.N, V=sum(len)), by=meta_cols]
 
         if (!is.null(hits) | !is.null(feature)){
-          if (!is.null(hits) & !is.null(feature)) stop('Cannot specify both hits and feature')
+          if (!is.null(hits) && !is.null(feature)) stop('Cannot specify both hits and feature')
           if (!is.null(hits)) {
             if (!methods::is(hits, c('featureHits', 'contextHits'))) stop('hits must be a featureHits or contextHits object (see the $search_features and $search_contexts methods)')
             if (methods::is(hits, 'featureHits')) {
@@ -378,7 +440,7 @@ tCorpus <- R6::R6Class("tCorpus",
           }
 
           if (!is.null(feature)) {
-            coldata = data.frame(doc_id = private$.data$doc_id, code = private$.data[[feature]])
+            coldata = data.frame(doc_id = self$tokens$doc_id, code = self$tokens[[feature]])
             coldata = coldata[!is.na(coldata$code),]
           }
 
@@ -418,11 +480,11 @@ tCorpus <- R6::R6Class("tCorpus",
 
         if (any(x == '*')) {
           if (only_context) {
-            out = unique(private$.data, by = c('doc_id', subcontext))
+            out = unique(self$tokens, by = c('doc_id', subcontext))
           } else {
-            out = data.table::copy(private$.data)
+            out = data.table::copy(self$tokens)
           }
-          if (with_i & !only_context) out[, i:=1:nrow(out)]
+          if (with_i && !only_context) out[, i:=1:nrow(out)]
           if (has_sub_query) {
             if (is.null(sub_out)) return(NULL)
             out = data.table::fintersect(out, sub_out)
@@ -432,25 +494,26 @@ tCorpus <- R6::R6Class("tCorpus",
         }
 
         ## prepare lookup table: set a (secondary) data.table index
-        if (!feature %in% indices(private$.data)) {
-          data.table::setindexv(private$.data, feature)
+        if (!feature %in% indices(self$tokens)) {
+
+          data.table::setindexv(self$tokens, feature)
           message(sprintf('created index for "%s" column', feature))
         }
 
         ## if not fixed (exact value matching), first lookup x as regex in unique values
         ## note that mem_lookup_terms is memoised
         if (!fixed) {
-          uval = if (is.factor(private$.data[[feature]])) levels(private$.data[[feature]]) else unique(private$.data[[feature]])
+          uval = if (is.factor(self$tokens[[feature]])) levels(self$tokens[[feature]]) else unique(self$tokens[[feature]])
           x = mem_lookup_terms(x, uval, ignore_case=ignore_case, raw_regex=raw_regex, batchsize=batchsize, useBytes=T, as_ascii=as_ascii)
         }
 
         if (length(x) == 0) return(NULL)
-        if (with_i & !only_context) {
-          i = na.omit(private$.data[list(x), on=feature, which=T])
-          out = private$.data[i,]
+        if (with_i && !only_context) {
+          i = na.omit(self$tokens[list(x), on=feature, which=T])
+          out = self$tokens[i,]
           out[,i:=i]
         } else {
-          out = private$.data[list(x), on=feature, which=F]
+          out = self$tokens[list(x), on=feature, which=F]
         }
         if (only_context) out = unique(out, by=c('doc_id',subcontext))
 
@@ -465,39 +528,38 @@ tCorpus <- R6::R6Class("tCorpus",
       },
 
       forget_memoise = function() forget_all_mem(),
-      indices = function() data.table::indices(private$.data),
-      clear_indices = function() data.table::setindex(private$.data, NULL)
+      indices = function() data.table::indices(self$tokens),
+      clear_indices = function() data.table::setindex(self$tokens, NULL)
    ),
 
+
+
    active = list(
-     n = function() nrow(private$.data),
-     n_meta = function() nrow(private$.meta),
+     n = function() nrow(self$tokens),
+     n_meta = function() nrow(self$meta),
      feature_names = function(e=NULL) {
        if (!is.null(e)) stop('Cannot change tcorpus$feature_names by assignment. Instead, use the set_name() function')
-       fnames = colnames(private$.data)[!colnames(private$.data) %in% c('doc_id','sentence','token_id')]
+       fnames = colnames(self$tokens)[!colnames(self$tokens) %in% c('doc_id','sentence','token_id')]
      },
      names = function(e=NULL) {
        if (!is.null(e)) stop('Cannot change tcorpus$names by assignment. Instead, use the set_name() function')
-       colnames(private$.data)
+       colnames(self$tokens)
      },
      meta_names = function(e=NULL) {
        if (!is.null(e)) stop('Cannot change tcorpus$meta_names by assignment. Instead, use the set_meta_name() function')
-       colnames(private$.meta)
+       colnames(self$meta)
      },
 
-
      doc_id_levels = function(mod=NULL) {
        if (!is.null(mod)){
-         private$.meta$doc_id = mod[match(levels(private$.data$doc_id), private$.meta$doc_id)]
-         levels(private$.data$doc_id) = mod
+         self$meta$doc_id = mod[match(levels(self$tokens$doc_id), self$meta$doc_id)]
+         levels(self$tokens$doc_id) = mod
        }
        levels(self$get('doc_id'))
      }
    )
 )
 
-
-
 #' S3 print for tCorpus class
 #'
 #' @param x a tCorpus object
@@ -513,11 +575,12 @@ print.tCorpus <- function(x, ...) {
   cat('tCorpus containing ', x$n, ' tokens',
       '\ngrouped by documents (n = ', x$n_meta, ')', sent_info,
       '\ncontains:',
-      '\n  - ', length(x$names), ' data column', if (length(x$names) > 1) 's', ':\t', paste(x$names, collapse=', '),
-      '\n  - ', length(x$meta_names), ' meta column', if (length(x$meta_names) > 1) 's', ': \t', paste(x$meta_names, collapse=', '),
+      '\n  - ', length(x$names), ' column', if (length(x$names) > 1) 's', ' in $tokens', ':\t', paste(x$names, collapse=', '),
+      '\n  - ', length(x$meta_names), ' column', if (length(x$meta_names) > 1) 's', ' in $meta', ':  \t', paste(x$meta_names, collapse=', '),
       '\n', sep='')
 }
 
+
 #' Refresh a tCorpus object using the current version of corpustools
 #'
 #' As an R6 class, tCorpus contains its methods within the class object (i.e. itself). Therefore, if you use a new version of corpustools with an older tCorpus object (e.g., stored as a .rds. file), then the methods are not automatically updated. You can then use refresh_tcorpus() to reinitialize the tCorpus object with the current version of corpustools.
@@ -530,8 +593,8 @@ print.tCorpus <- function(x, ...) {
 #' refresh_tcorpus(tc)
 #' @export
 refresh_tcorpus <- function(tc){
-  tCorpus$new(data=tc$get(),
-              meta=tc$get_meta())
+  tCorpus$new(tokens = data.table::copy(tc$get(keep_df = T)),
+              meta = data.table::copy(tc$get_meta(keep_df = T)))
 }
 
 rebuild_tcorpus <- function(tc) {
@@ -589,7 +652,7 @@ as.tcorpus.default <- function(x, ...) stop('x has to be a tCorpus object')
 
 is_tcorpus <- function(x, allow_stc=F){
   if (!class(x)[1] %in% c('tCorpus', 'shattered_tCorpus')) stop('not a tCorpus object')
-  if (is_shattered(x) & !allow_stc) stop('function not implemented for shattered_tCorpus')
+  if (is_shattered(x) && !allow_stc) stop('function not implemented for shattered_tCorpus')
   TRUE
 }
 
@@ -599,7 +662,7 @@ is_shattered <- function(x) methods::is(x, 'shattered_tCorpus')
 
 safe_selection <- function(d, selection){
   if (any(is.na(selection))) stop('selection cannot contain NA')
-  if (!methods::is(selection, 'numeric') & !methods::is(selection,'logical')) stop('selection has to be either a logical vector or a numerical vector (indices for TRUE values)')
+  if (!methods::is(selection, 'numeric') && !methods::is(selection,'logical')) stop('selection has to be either a logical vector or a numerical vector (indices for TRUE values)')
   if (methods::is(selection, 'numeric')) selection = 1:nrow(d) %in% selection
   selection
 }
