@@ -8,7 +8,8 @@
 #' @description
 #' Get the token and meta data.
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 active method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{get(columns=NULL, keep_df=F, as.df=F, subset=NULL, doc_id=NULL, token_id=NULL, safe_copy=T)}
@@ -58,7 +59,8 @@ NULL
 #'
 #' Depending on the purpose, the context of an analysis can be the document level or sentence level. the tCorpus$context() method offers a convenient way to get the context id of tokens for different settings.
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{data(context_level = c('document','sentence'), with_labels = T)}
@@ -81,11 +83,31 @@ NULL
 
 ############### MODIFY DATA
 
+#' Designate column as columns with special meaning (token, lemma, POS, relation, parent)
+#'
+#' \strong{Usage:}
+#'
+#' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
+#'
+#' \preformatted{set_special(token=NULL, lemma=NULL, POS=NULL, relation=NULL, parent=NULL)}
+#'
+#' @param token Name of the column that will be designated as the token, and renamed to 'token'
+#' @param lemma Name of the column that will be designated as the lemma of the token, and renamed to 'lemma'
+#' @param pos Name of the column that will be designated as the part-of-speech tag of the token, and renamed to 'POS'
+#' @param relation Name of the column that will be designated as the dependency relation of the token to its parent, and renamed to 'relation'
+#' @param parent Name of the column that will be designated as the parent of the token, and renamed to 'parent'
+#'
+#' @name tCorpus$set_special
+#' @aliases set_special
+NULL
+
+
 #' Modify the token and meta data.tables of a tCorpus
 #'
 #' Modify the token/meta data.table by setting the values of one (existing or new) column. The subset argument can be used to modify only subsets of columns, and can be a logical vector (select TRUE rows), numeric vector (indices of TRUE rows) or logical expression (e.g. pos == 'noun'). If a new column is made whie using a subset, then the rows outside of the selection are set to NA.
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{set(column, value, subset)}
@@ -124,7 +146,8 @@ NULL
 #'
 #' For factor columns, the levels can be changed directly (and by reference). This is particularly usefull for fast preprocessing (e.g., making tokens lowercase, )
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{set_levels(column, levels)}
@@ -147,7 +170,8 @@ NULL
 
 #' Change column names of data and meta data
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{set_name(oldname, newname)}
@@ -175,7 +199,8 @@ NULL
 
 #' Delete column from the data and meta data
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{delete_columns(cnames)}
@@ -215,7 +240,8 @@ NULL
 #'
 #' Note that you can also use the \link{tCorpus$feature_subset} method if you want to filter out low/high frequency tokens, but do not want to delete the rows in the tCorpus.
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{
