@@ -1,6 +1,7 @@
 #' Preprocess feature
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{
@@ -18,10 +19,15 @@
 #' @param as_ascii convert characters to ascii. This is particularly usefull for dealing with special characters.
 #' @param remove_punctuation remove (i.e. make NA) any features that are \emph{only} punctuation (e.g., dots, comma's)
 #' @param remove_stopwords remove (i.e. make NA) stopwords. (!) Make sure to set the language argument correctly.
+#' @param remove_numbers remove features that are only numbers
 #' @param use_stemming reduce features (tokens) to their stem
 #' @param language The language used for stopwords and stemming
 #' @param min_freq an integer, specifying minimum token frequency.
 #' @param min_docfreq an integer, specifying minimum document frequency.
+#' @param max_freq an integer, specifying minimum token frequency.
+#' @param max_docfreq an integer, specifying minimum document frequency.
+#' @param min_char an integer, specifying minimum number of characters in a term
+#' @param max_char an integer, specifying maximum number of characters in a term
 #'
 #' @name tCorpus$preprocess
 #' @aliases preprocess
@@ -41,9 +47,9 @@
 #' tc$preprocess('token', 'preprocessed_4', ngrams = 3)
 #'
 #' tc$get()
-tCorpus$set('public', 'preprocess', function(column, new_column=column, lowercase=T, ngrams=1, ngram_context=c('document', 'sentence'), as_ascii=F, remove_punctuation=T, remove_stopwords=F, use_stemming=F, language='english', min_freq=NULL, min_docfreq=NULL) {
+tCorpus$set('public', 'preprocess', function(column='token', new_column='feature', lowercase=T, ngrams=1, ngram_context=c('document', 'sentence'), as_ascii=F, remove_punctuation=T, remove_stopwords=F, remove_numbers=F, use_stemming=F, language='english', min_freq=NULL, min_docfreq=NULL, max_freq=NULL, max_docfreq=NULL, min_char=NULL, max_char=NULL) {
   column = match.arg(column, self$names)
-  invisible(preprocess_feature(self, column=column, new_column=new_column, lowercase=lowercase, ngrams=ngrams, ngram_context=ngram_context, as_ascii=as_ascii, remove_punctuation=remove_punctuation, remove_stopwords=remove_stopwords, use_stemming=use_stemming, language=language, min_freq=min_freq, min_docfreq=min_docfreq))
+  invisible(preprocess_feature(self, column=column, new_column=new_column, lowercase=lowercase, ngrams=ngrams, ngram_context=ngram_context, as_ascii=as_ascii, remove_punctuation=remove_punctuation, remove_stopwords=remove_stopwords, remove_numbers=remove_numbers, use_stemming=use_stemming, language=language, min_freq=min_freq, min_docfreq=min_docfreq, max_freq=max_freq, max_docfreq=max_docfreq, min_char=min_char, max_char=max_char))
 })
 
 #' Filter features
@@ -53,7 +59,8 @@ tCorpus$set('public', 'preprocess', function(column, new_column=column, lowercas
 #'
 #' Just as in subset, it is easy to use objects and functions in the filter, including the special functions for using term frequency statistics (see documentation for \link{tCorpus$subset}).
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{feature_subset(column, new_column, subset)}
@@ -81,7 +88,7 @@ tCorpus$set('public', 'feature_subset', function(column, new_column=column, subs
     return(selfcopy)
   }
 
-  if (is(subset, 'numeric')) subset = 1:self$n %in% subset ## this can be the case if a vector of indices is passed to subset (which is not a valid call, but is allowed for convenience because it is a common way of subsetting)
+  if (methods::is(subset, 'numeric')) subset = 1:self$n %in% subset ## this can be the case if a vector of indices is passed to subset (which is not a valid call, but is allowed for convenience because it is a common way of subsetting)
 
   .subset = if (inverse) !subset else subset
 
@@ -96,18 +103,17 @@ tCorpus$set('public', 'feature_subset', function(column, new_column=column, subs
   invisible(self)
 })
 
-
-preprocess_feature <- function(tc, column, new_column, lowercase=T, ngrams=1, ngram_context=c('document', 'sentence'), as_ascii=F, remove_punctuation=T, remove_stopwords=F, use_stemming=F, language='english', min_freq=NULL, min_docfreq=NULL){
+preprocess_feature <- function(tc, column, new_column, lowercase=T, ngrams=1, ngram_context=c('document', 'sentence'), as_ascii=F, remove_punctuation=T,remove_stopwords=F, remove_numbers=F, use_stemming=F, language='english', min_freq=NULL, min_docfreq=NULL, max_freq=NULL, max_docfreq=NULL, min_char=NULL, max_char=NULL){
   is_tcorpus(tc, T)
 
   feature = tc$get(column)
   if (!methods::is(feature, 'factor')) feature = factor(feature)
 
-  if (ngrams == 1 & is.null(min_docfreq)) {
-    .feature = preprocess_tokens(feature, context=NA, language=language, use_stemming=use_stemming, lowercase=lowercase, as_ascii=as_ascii, remove_punctuation=remove_punctuation, remove_stopwords=remove_stopwords, min_freq=min_freq, min_docfreq=min_docfreq)
+  if (ngrams == 1 && is.null(min_docfreq)) {
+    .feature = preprocess_tokens(feature, context=NA, language=language, use_stemming=use_stemming, lowercase=lowercase, as_ascii=as_ascii, remove_punctuation=remove_punctuation, remove_stopwords=remove_stopwords, remove_numbers=remove_numbers, min_freq=min_freq, min_docfreq=min_docfreq, max_freq=max_freq, max_docfreq=max_docfreq, min_char=min_char, max_char=max_char)
   } else {
     context = tc$context(context_level=ngram_context, with_labels = F)
-    .feature = preprocess_tokens(feature, context=context, language=language, use_stemming=use_stemming, lowercase=lowercase, ngrams = ngrams, as_ascii=as_ascii, remove_punctuation=remove_punctuation, remove_stopwords=remove_stopwords, min_freq=min_freq, min_docfreq=min_docfreq)
+    .feature = preprocess_tokens(feature, context=context, language=language, use_stemming=use_stemming, lowercase=lowercase, ngrams = ngrams, as_ascii=as_ascii, remove_punctuation=remove_punctuation, remove_stopwords=remove_stopwords, remove_numbers=remove_numbers, min_freq=min_freq, min_docfreq=min_docfreq, max_freq=max_freq, max_docfreq=max_docfreq, min_char=min_char, max_char=max_char)
   }
   tc$set(column = new_column, value = .feature)
 }
@@ -125,8 +131,14 @@ preprocess_feature <- function(tc, column, new_column, lowercase=T, ngrams=1, ng
 #' @param as_ascii Logical. If TRUE, tokens will be forced to ascii
 #' @param remove_punctuation Logical. if TRUE, punctuation is removed
 #' @param remove_stopwords Logical. If TRUE, stopwords are removed (Make sure to specify the right language!)
+#' @param remove_numbers remove features that are only numbers
 #' @param min_freq an integer, specifying minimum token frequency.
 #' @param min_docfreq an integer, specifying minimum document frequency.
+#' @param max_freq an integer, specifying minimum token frequency.
+#' @param max_docfreq an integer, specifying minimum document frequency.
+#' @param min_char an integer, specifying minimum number of characters in a term
+#' @param max_char an integer, specifying maximum number of characters in a term
+#' @param ngram_skip_empty if ngrams are used, determines whether empty (filtered out) terms are skipped (i.e. c("this", NA, "test"), becomes "this_test") or
 #'
 #' @examples
 #' tokens = c('I', 'am', 'a', 'SHORT', 'example', 'sentence', '!')
@@ -138,64 +150,94 @@ preprocess_feature <- function(tc, column, new_column, lowercase=T, ngrams=1, ng
 #' preprocess_tokens(tokens, remove_stopwords = TRUE, use_stemming = TRUE)
 #' preprocess_tokens(tokens, context = NA, ngrams = 3)
 #' @export
-preprocess_tokens <- function(x, context=NULL, language='english', use_stemming=F, lowercase=T, ngrams=1, replace_whitespace=T, as_ascii=F, remove_punctuation=T, remove_stopwords=F, min_freq=NULL, min_docfreq=NULL){
+preprocess_tokens <- function(x, context=NULL, language='english', use_stemming=F, lowercase=T, ngrams=1, replace_whitespace=T, as_ascii=F, remove_punctuation=T, remove_stopwords=F, remove_numbers=F, min_freq=NULL, min_docfreq=NULL, max_freq=NULL, max_docfreq=NULL, min_char=NULL, max_char=NULL, ngram_skip_empty=T){
   language = match.arg(language, choices=c('danish','dutch','english','finnish','french','german','hungarian','italian','norwegian','porter','portuguese','romanian','russian','spanish','swedish','turkish'))
   if (!methods::is(x, 'factor')) x = fast_factor(x)
   if (replace_whitespace) levels(x) = gsub(' ', '_', levels(x), fixed=T)
   if (lowercase) levels(x) = tolower(levels(x))
-  #if (as_ascii) levels(x) = iconv(levels(x), to='ASCII//TRANSLIT')
   if (as_ascii) {
     levels(x) = stringi::stri_trans_general(levels(x),"any-latin")
     levels(x) = stringi::stri_trans_general(levels(x),"latin-ascii")
   }
   if (remove_stopwords) levels(x)[levels(x) %in% get_stopwords(language)] = NA
-  if (remove_punctuation) levels(x)[!grepl("[[:alnum:]]", levels(x))] = NA
+
+  if (remove_punctuation && remove_numbers) levels(x)[!grepl("[[:alpha:]]", levels(x))] = NA
+  if (remove_punctuation && !remove_numbers) levels(x)[!grepl("[[:alnum:]]", levels(x))] = NA
+  if (!remove_punctuation && remove_numbers) levels(x)[!grepl("[[:alpha:][:punct:]]", levels(x))] = NA
+
   if (use_stemming) levels(x) = SnowballC::wordStem(levels(x), language=language)
 
   if (ngrams > 1) {
     if (is.null(context)) stop('For ngrams, the "context" argument has to be specified. If no context is available, "context" can be NA')
-    x = grouped_ngrams(x, context, ngrams)
+    x = grouped_ngrams(x, context, ngrams, skip_empty=ngram_skip_empty)
   }
 
-  if (!is.null(min_docfreq)) {
+  if (!is.null(min_docfreq) | !is.null(max_docfreq)) {
+    if (is.null(min_docfreq)) min_docfreq = -Inf
+    if (is.null(max_docfreq)) max_docfreq = Inf
     if (is.null(context)) {
       freq_table = unique(data.frame(doc_id=1, x=x))
     } else {
       freq_table = unique(data.frame(doc_id=context, x=x))
     }
     freq_table = table(droplevels(freq_table$x))
-    levels(x)[!levels(x) %in% x_filter(freq_table, min=min_docfreq)] = NA
+    levels(x)[!levels(x) %in% x_filter(freq_table, min=min_docfreq, max=max_docfreq)] = NA
   }
 
-  if (!is.null(min_freq)) {
+  if (!is.null(min_freq) | !is.null(max_freq)) {
+    if (is.null(min_freq)) min_freq = -Inf
+    if (is.null(max_freq)) max_freq = Inf
+
     freq_table = table(droplevels(x))
-    levels(x)[!levels(x) %in% x_filter(freq_table, min=min_freq)] = NA
+    levels(x)[!levels(x) %in% x_filter(freq_table, min=min_freq, max=max_freq)] = NA
+  }
+
+  if (!is.null(min_char) | !is.null(max_char)) {
+    if (is.null(min_char)) min_char = -Inf
+    if (is.null(max_char)) max_char = Inf
+    len = stringi::stri_length(levels(x))
+    levels(x)[len < min_char | len > max_char] = NA
   }
 
   x
 }
 
-
 create_ngrams <- function(tokens, group, n, label=T, sep = '/', empty='') {
   if (!length(tokens) == length(group)) stop("tokens has to be of same length as group")
-  ng = .Call('_corpustools_ngrams', PACKAGE = 'corpustools', tokens, group, n, sep, empty)
+
+  ng = ngrams_cpp(tokens, group, n, sep, empty)
   ng = fast_factor(ng)
   if (label) return(ng) else return(as.numeric(ng))
 }
 
-grouped_ngrams <- function(tokens, group, n, filter=rep(T, length(tokens)), label=T){
+grouped_ngrams <- function(tokens, group, n, filter=rep(T, length(tokens)), label=T, skip_empty=T){
   filter = filter & !is.na(tokens)
-  tokens = tokens[filter]
-  group = if (length(group) == 1) rep(group, length(tokens)) else group[filter]
+
+  if (skip_empty) {
+    tokens = tokens[filter]
+    group = if (length(group) == 1) rep(group, length(tokens)) else group[filter]
+  } else {
+    if (methods::is(tokens, 'factor')) levels(tokens) = union(levels(tokens), '')
+    group = if (length(group) == 1) rep(group, length(tokens)) else group
+  }
 
   if (label) {
-    ngrams = as.factor(rep(NA, length(filter)))
-    ng = create_ngrams(tokens, group, n, label=label)
-    levels(ngrams) = levels(ng)
-    ngrams[which(filter)] = ng
+    if (skip_empty) {
+      ngrams = as.factor(rep(NA, length(filter)))
+      ng = create_ngrams(tokens, group, n, label=label)
+      levels(ngrams) = levels(ng)
+      ngrams[which(filter)] = ng
+    } else {
+      ngrams = create_ngrams(tokens, group, n, label=label)
+    }
   } else {
-    ngrams = vector('numeric', length(filter))
-    ngrams[which(filter)] = create_ngrams(tokens, group, n, label=label)
+    if (skip_empty) {
+      ngrams = vector('numeric', length(filter))
+      ngrams[which(filter)] = create_ngrams(tokens, group, n, label=label)
+    } else {
+      ngrams = create_ngrams(tokens, group, n, label=label)
+      ngrams[!filter] = NA
+    }
   }
   ngrams
 }
@@ -219,3 +261,4 @@ get_stopwords <- function(lang){
   corpustools::stopwords_list[[lang]]
 }
 
+
