@@ -3,9 +3,10 @@
 #' @description
 #' Search tokens in a tokenlist using Lucene-like queries. For a detailed explanation of the query language, see the details below.
 #'
-#' @section Usage:
-#' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
+#' \strong{Usage:}
+#'
+#' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #' \preformatted{
 #' search_features(query, code = NA, feature = 'token',
 #'                 mode = c('unique_hits','features'), verbose = F)
@@ -160,12 +161,11 @@ tCorpus$set('public', 'search_features', function(query, code=NULL, feature='tok
 #' @description
 #' Add a column to the token data that contains a code (the query label) for tokens that match the query (see \link{tCorpus$search_features}).
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
-#' \preformatted{
-#' code_features(query, code=NULL, feature='token', column='code', ...)
-#' }
+#' \preformatted{code_features(query, code=NULL, feature='token', column='code', ...)}
 #'
 #' @param query A character string that is a query. See \link{search_features} for documentation of the query language.
 #' @param code The code given to the tokens that match the query (usefull when looking for multiple queries). Can also put code label in query with # (see details)
@@ -190,7 +190,7 @@ tCorpus$set('public', 'code_features', function(query, code=NULL, feature='token
   code = 1:length(query)
   hits = search_features(self, query, code=code, feature=feature, mode='features', context_level=context_level, keep_longest=keep_longest, as_ascii=as_ascii, verbose=verbose)
 
-  .i = self$token_id(doc_id = hits$hits$doc_id, token_id = hits$hits$token_id)
+  .i = self$get_token_id(doc_id = hits$hits$doc_id, token_id = hits$hits$token_id)
   .value = codelabel[as.numeric(hits$hits$code)]
   self$set(column=column, subset=.i, value=.value, subset_value=F)
 
@@ -212,7 +212,8 @@ tCorpus$set('public', 'code_features', function(query, code=NULL, feature='token
 #' @description
 #' Search features (see \link{tCorpus$search_features}) and replace features with a new value
 #'
-#' @section Usage:
+#' \strong{Usage:}
+#'
 #' ## R6 method for class tCorpus. Use as tc$method (where tc is a tCorpus object).
 #'
 #' \preformatted{
@@ -228,7 +229,7 @@ tCorpus$set('public', 'code_features', function(query, code=NULL, feature='token
 #' @aliases search_recode
 tCorpus$set('public', 'search_recode', function(feature, new_value, query, ...){
   hits = search_features(self, query, feature=feature, mode='features', ...)
-  .i = self$token_id(doc_id = hits$hits$doc_id, token_id = hits$hits$token_id)
+  .i = self$get_token_id(doc_id = hits$hits$doc_id, token_id = hits$hits$token_id)
   .new_value = new_value
   self$set(feature, .new_value, subset = .i)
   invisible(self)
@@ -248,14 +249,17 @@ search_features <- function(tc, query, code=NULL, feature='token', mode = c('uni
 
   for (i in 1:length(query)) {
     if (verbose) cat(as.character(codelabel[i]), '\n')
-    q = parse_query(as.character(query[i]))
+    q = parse_query_cpp(as.character(query[i]))
+
     h = recursive_search(tc, q, subcontext=subcontext, feature=feature, mode = mode, keep_longest=keep_longest, as_ascii=as_ascii)
+
     if (!is.null(h)) {
       h[, code := codelabel[i]]
       hits[[i]] = h
     }
   }
-  hits = data.table::rbindlist(hits)
+  t = tc$tokens
+  hits = data.table::rbindlist(hits, fill = T)
 
   if (nrow(hits) > 0) {
     data.table::setnames(hits, feature, 'feature')
